@@ -7,14 +7,14 @@ class AccumulateData(object):
     def __init__(self):
         """ Initialize data instance """
         prefix = 'A concrete implementation of AccumulateData must have '
-        if not hasattr(self,'stopping_criterion'):
-            raise ParameterError(prefix + 'self.stopping_criterion (a StoppingCriterion)')
+        if not hasattr(self,'stopping_crit'):
+            raise ParameterError(prefix + 'self.stopping_crit (a StoppingCriterion)')
         if not hasattr(self,'integrand'):
             raise ParameterError(prefix + 'self.integrand (an Integrand)')
-        if not hasattr(self,'measure'):
-            raise ParameterError(prefix + 'self.measure (a TrueMeasure)')
-        if not hasattr(self,'distribution'):
-            raise ParameterError(prefix + 'self.distribution (a DiscreteDistribution)')
+        if not hasattr(self,'true_measure'):
+            raise ParameterError(prefix + 'self.true_measure (a TrueMeasure)')
+        if not hasattr(self,'discrete_distrib'):
+            raise ParameterError(prefix + 'self.discrete_distrib (a DiscreteDistribution)')
         if not hasattr(self, 'solution'):
             raise ParameterError(prefix + 'self.solution')
         if not hasattr(self, 'n_total'):
@@ -27,9 +27,8 @@ class AccumulateData(object):
         raise MethodImplementationError(self, 'update_data')
 
     def __repr__(self):
-        string = "Solution: %-15.4f\n" % (self.solution)
-        for qmc_obj in [self.integrand, self.distribution, self.measure, self.stopping_criterion]:
+        string = _univ_repr(self, 'AccumulateData', self.parameters + ['time_integrate'])
+        for qmc_obj in [self.stopping_crit, self.integrand, self.true_measure, self.discrete_distrib]:
             if qmc_obj:
-                string += str(qmc_obj)+'\n'
-        string += _univ_repr(self, 'AccumulateData', self.parameters + ['time_integrate'])
+                string += '\n'+str(qmc_obj)
         return string
