@@ -1,7 +1,7 @@
 from .._discrete_distribution import DiscreteDistribution
 from ...util import ParameterError, ParameterWarning
 from numpy import *
-from os.path import dirname, abspath
+from os.path import dirname, abspath, isfile
 import warnings
 
 
@@ -10,24 +10,21 @@ class Lattice(DiscreteDistribution):
     Quasi-Random Lattice nets in base 2.
 
     >>> l = Lattice(2,seed=7)
+    >>> l.gen_samples(4)
+    array([[0.04386058, 0.58727432],
+           [0.54386058, 0.08727432],
+           [0.29386058, 0.33727432],
+           [0.79386058, 0.83727432]])
+    >>> l.gen_samples(1)
+    array([[0.04386058, 0.58727432]])
     >>> l
     Lattice (DiscreteDistribution Object)
-        dimension       2^(1)
+        d               2^(1)
+        dvec            [0 1]
         randomize       1
         order           natural
-        seed            7
-        mimics          StdUniform
-    >>> l.gen_samples(4)
-    array([[0.076, 0.78 ],
-           [0.576, 0.28 ],
-           [0.326, 0.53 ],
-           [0.826, 0.03 ]])
-    >>> l.set_dimension(3)
-    >>> l.gen_samples(n_min=4,n_max=8)
-    array([[0.563, 0.098, 0.353],
-           [0.063, 0.598, 0.853],
-           [0.813, 0.848, 0.103],
-           [0.313, 0.348, 0.603]])
+        entropy         7
+        spawn_key       ()
     >>> Lattice(dimension=2,randomize=False,order='natural').gen_samples(4, warn=False)
     array([[0.  , 0.  ],
            [0.5 , 0.5 ],
@@ -71,21 +68,29 @@ class Lattice(DiscreteDistribution):
         ACM Transactions on Mathematical Software. 42. 10.1145/2754929.
     """
 
-    parameters = ['dimension','randomize','order','seed','mimics']
-
-    def __init__(self, dimension=1, randomize=True, order='natural', seed=None, z_path=None):
+    def __init__(self, dimension=1, randomize=True, order='natural', seed=None, 
+        generating_vector='lattice_vec.3600.20.npy', d_max=None, m_max=None):
         """
         Args:
-            dimension (int): dimension of samples
+           dimension (int or ndarray): dimension of the generator. 
+                If an int is passed in, use sequence dimensions [0,...,dimensions-1].
+                If a ndarray is passed in, use these dimension indices in the sequence. 
             randomize (bool): If True, apply shift to generated samples. \
                 Note: Non-randomized lattice sequence includes the origin.
             order (str): 'linear', 'natural', or 'mps' ordering.
-            seed (int): seed the random number generator for reproducibility
-            z_path (str): path to generating vector. 
-                z_path should be formatted like 'lattice_vec.3600.20.npy' where 'name.d_max.m_max.npy' 
-                and d_max is the maximum dimenion and 2^m_max is the max number samples supported
+            seed (None or int or numpy.random.SeedSeq): seed the random number generator for reproducibility
+            generating_vector (ndarray or str): generating matrix or path to generating matricies. 
+                ndarray should have shape (d_max).  
+                a string generating_vector should be formatted like 
+                'lattice_vec.3600.20.npy' where 'name.d_max.m_max.npy' 
+            d_max (int): maximum dimension
+            m_max (int): 2^m_max is the max number of supported samples
+        
+        Note:
+            d_max and m_max are required if generating_vector is a ndarray.
+            If generating_vector is an string (path), d_max and m_max can be taken from the file name if None
         """
-        # set generating matrix
+        self.parameters = ['dvec','randomize','order']
         self.randomize = randomize
         self.order = order.lower()
         if self.order == 'natural':
@@ -96,24 +101,30 @@ class Lattice(DiscreteDistribution):
             self.gen = self._mps
         else: 
             raise Exception("Lattice requires natural, linear, or mps ordering.")
-        if not z_path:
-            self.d_max = 3600
-            self.m_max = 20
-            self.msb = True
-            self.z_full = load(dirname(abspath(__file__))+'/generating_vectors/lattice_vec.3600.20.npy').astype(uint64)
+        if isinstance(generating_vector,ndarray):
+            self.z_og = generating_vector
+            if d_max is None or m_max is None:
+                raise ParameterError("d_max and m_max must be supplied when generating_vector is a ndarray")
+            self.d_max = d_max
+            self.m_max = m_max
+        elif isinstance(generating_vector,str):
+            root = dirname(abspath(__file__))+'/generating_vectors/'
+            if isfile(root+generating_vector):
+                self.z_og = load(root+generating_vector).astype(uint64)
+            elif isfile(generating_vector):
+                self.z_og = load(generating_vector).astype(uint64)
+            else:
+                raise ParameterError("generating_vector '%s' not found."%generating_vector)
+            parts = generating_vector.split('.')
+            self.d_max = int(parts[-3])
+            self.m_max = int(parts[-2])
         else:
-            if not isfile(z_path):
-                raise ParameterError('z_path `' + z_path + '` not found. ')
-            self.z_full = load(z_path).astype(uint64)
-            f = z_path.split('/')[-1]
-            f_lst = f.split('.')
-            self.d_max = int(f_lst[1])
-            self.m_max = int(f_lst[2])
-        self.set_dimension(dimension)
-        self.set_seed(seed)
-        self.low_discrepancy = True
+            raise ParameterError("generating_vector should a ndarray or file path string")
         self.mimics = 'StdUniform'
-        super(Lattice,self).__init__()
+        self.low_discrepancy = True
+        super(Lattice,self).__init__(dimension,seed)
+        self.z = self.z_og[self.dvec]
+        self.shift = self.rng.uniform(size=int(self.d))
     
     def _mps(self, n_min, n_max):
         """ Magic Point Shop Lattice generator. """
@@ -126,16 +137,35 @@ class Lattice(DiscreteDistribution):
         x = x_lat_full[cut1:cut2,:]
         return x
 
-    def _gail_linear(self, n_min, n_max):
-        """ Gail lattice generator in linear order. """
-        nelem = n_max - n_min
-        if n_min == 0:
-            y = arange(0, 1, 1 / nelem).reshape((nelem, 1))
+    def _gen_block_linear(self, m_next, first=True):
+        n = int(2**m_next)
+        if first:
+            y = arange(0, 1, 1 / n).reshape((n, 1))
         else:
-            y = arange(1 / n_max, 1, 2 / n_max).reshape((nelem, 1))
+            y = arange(1 / n, 1, 2 / n).reshape((n, 1))
         x = outer(y, self.z) % 1
         return x
 
+    def _gail_linear(self, n_min, n_max):
+        """ Gail lattice generator in linear order. """
+        m_low = int(floor(log2(n_min))) + 1 if n_min > 0 else 0
+        m_high = int(ceil(log2(n_max)))
+        if n_min == 0:
+            return self._gen_block_linear(m_high, first=True)
+        else:
+            n = 2**(m_low)
+            y = arange(1 / n, 1, 2 / n).reshape((int(n/2), 1))
+            for m in range(m_low, m_high):
+                n = 2**(m)
+                y_next = arange(1 / n, 1, 2 / n).reshape((int(n/2), 1))
+                temp = zeros((n, 1))
+                temp[0::2] = y
+                temp[1::2] = y_next
+                y = temp
+
+            x = outer(y, self.z) % 1
+            return x
+
     def _gail_natural(self, n_min, n_max):
         m_low = floor(log2(n_min)) + 1 if n_min > 0 else 0
         m_high = ceil(log2(n_max))
@@ -168,7 +198,7 @@ class Lattice(DiscreteDistribution):
         x = outer(self._vdc(n)+1./(2*n_min),self.z)%1 if n_min>0 else outer(self._vdc(n),self.z)%1
         return x
 
-    def gen_samples(self, n=None, n_min=0, n_max=8, warn=True):
+    def gen_samples(self, n=None, n_min=0, n_max=8, warn=True, return_unrandomized=False):
         """
         Generate lattice samples
 
@@ -177,7 +207,8 @@ class Lattice(DiscreteDistribution):
                 Otherwise use the n_min and n_max explicitly supplied as the following 2 arguments
             n_min (int): Starting index of sequence.
             n_max (int): Final index of sequence.
-            return_non_random (bool): return both the samples with and without randomization
+            return_unrandomized (bool): return samples without randomization as 2nd return value. 
+                Will not be returned if randomize=False. 
 
         Returns:
             ndarray: (n_max-n_min) x d (dimension) array of samples
@@ -190,38 +221,32 @@ class Lattice(DiscreteDistribution):
         if n:
             n_min = 0
             n_max = n
+        if return_unrandomized and self.randomize==False:
+            raise ParameterError("return_unrandomized=True only applies when when randomize=True.")
         if n_min == 0 and self.randomize==False and warn:
             warnings.warn("Non-randomized lattice sequence includes the origin",ParameterWarning)
         if n_max > 2**self.m_max:
             raise ParameterError('Lattice generating vector supports up to %d samples.'%(2**self.m_max))
         x = self.gen(n_min,n_max)
         if self.randomize:
-            x = self.apply_randomization(x)
-        return x
-    
-    def apply_randomization(self, x):
-        """
-        Apply a digital shift to the samples. 
-        
-        Args:
-            x (ndarray): un-randomized samples to be digitally shifted. 
-        
-        Return:
-            ndarray: x with digital shift aplied.
-        """
-        x_rand = (x + self.shift)%1
-        return x_rand
+            xr = (x + self.shift)%1
+        if self.randomize==False:
+            return x
+        elif return_unrandomized:
+            return xr, x
+        else:
+            return xr
 
-    def set_seed(self, seed):
-        """ See abstract method. """
-        self.seed = seed if seed else random.randint(0, 100000, dtype=uint64)
-        random.seed(self.seed)
-        self.shift = random.rand(int(self.dimension))
-        
-    def set_dimension(self, dimension):
-        """ See abstract method. """
-        self.dimension = dimension
-        if self.dimension > self.d_max:
-            raise ParameterError('Lattice requires dimension <= %d'%self.d_max)
-        self.z = self.z_full[:self.dimension]
-        self.shift = random.rand(int(self.dimension))
\ No newline at end of file
+    def pdf(self, x):
+        """ pdf of a standard uniform """
+        return ones(x.shape[0], dtype=float)
+    
+    def _spawn(self, child_seed, dimension):
+        return Lattice(
+                dimension=dimension,
+                randomize=self.randomize,
+                order=self.order,
+                seed=child_seed,
+                generating_vector=self.z_og,
+                d_max=self.d_max,
+                m_max=self.m_max)
