@@ -1,6 +1,8 @@
 from .asian_option import AsianOption
 from .european_option import EuropeanOption
 from .keister import Keister
-from .linear import Linear
+from .linear0 import Linear0
 from .custom_fun import CustomFun
-from .ml_call_options import MLCallOptions
\ No newline at end of file
+from .ml_call_options import MLCallOptions
+from .box_integral import BoxIntegral
+from .sobol_indices import SobolIndices
\ No newline at end of file
