@@ -1,5 +1,5 @@
 from ._integrand import Integrand
-from ..discrete_distribution import Sobol
+from ..discrete_distribution import DigitalNetB2
 from ..true_measure import Gaussian
 from ..util import ParameterError
 from numpy import *
@@ -10,10 +10,8 @@ class MLCallOptions(Integrand):
     """
     Various call options from finance using Milstein discretization with $2^l$ timesteps on level $l$.
 
-    >>> dd = Sobol(seed=7)
-    >>> m = Gaussian(dd)
-    >>> mlco = MLCallOptions(m)
-    >>> mlco
+    >>> mlco_original = MLCallOptions(DigitalNetB2(seed=7))
+    >>> mlco_original
     MLCallOptions (Integrand Object)
         option          european
         sigma           0.200
@@ -21,15 +19,14 @@ class MLCallOptions(Integrand):
         r               0.050
         t               1
         b               85
-    >>> y = 0
-    >>> for level in range(4):
-    ...     new_dim = mlco._dim_at_level(level)
-    ...     m.set_dimension(new_dim)
-    ...     x = dd.gen_samples(2**10)
-    ...     sums,cost = mlco.f(x,l=level)
-    ...     y += sums[0]/2**10
-    >>> y
-    10.39...
+        level           0
+    >>> mlco_ml_dims = mlco_original.spawn(levels=arange(4))
+    >>> yml = 0
+    >>> for mlco in mlco_ml_dims:
+    ...     x = mlco.discrete_distrib.gen_samples(2**10)
+    ...     yml += mlco.f(x).mean()
+    >>> yml
+    10.393...
 
     References:
 
@@ -38,30 +35,32 @@ class MLCallOptions(Integrand):
     http://people.maths.ox.ac.uk/~gilesm/files/mcqmc06.pdf.
     """
 
-    parameters = ['option', 'sigma', 'k', 'r', 't', 'b']
-
-    def __init__(self, measure, option='european', volatility=.2,
-        start_strike_price=100., interest_rate=.05, t_final=1.):
+    def __init__(self, sampler, option='european', volatility=.2,
+        start_strike_price=100., interest_rate=.05, t_final=1., _level=0):
         """
         Args:
-            measure (TrueMeasure): A BrownianMotion TrueMeasure object
+            sampler (DiscreteDistribution/TrueMeasure): A 
+                discrete distribution from which to transform samples or a
+                true measure by which to compose a transform
             option_type (str): type of option in ["European","Asian"]
             volatility (float): sigma, the volatility of the asset
             start_strike_price (float): S(0), the asset value at t=0, and K, the strike price. \
                 Assume start_price = strike_price
             interest_rate (float): r, the annual interest rate
             t_final (float): exercise time
+            _level (int): for internal use only, users should not set this parameter. 
+
         """
-        if not (isinstance(measure,Gaussian) and (measure.mu==0).all() and (measure.sigma==eye(measure.d)).all()):
-            raise ParameterError('AsianOption measure must be a Gaussian instance with mean 0 and variance 1')
+        self.parameters = ['option', 'sigma', 'k', 'r', 't', 'b', 'level']
+        self.sampler = sampler
+        self.true_measure = Gaussian(self.sampler, mean=0, covariance=1)
+        self.discrete_distrib = self.true_measure.discrete_distrib
         options = ['european','asian']
         self.option = option.lower()
         if self.option not in options:
             raise ParameterError('option type must be one of\n\t%s'%str(options))
-        self.measure = measure
-        self.distribution = self.measure.distribution
-        if self.distribution.low_discrepancy and self.option=='asian':
-            raise ParameterError('MLCallOptions does not support LD sequence for Asian Option')
+        #if self.discrete_distrib.low_discrepancy and self.option=='asian':
+        #    raise ParameterError('MLCallOptions does not support LD sequence for Asian Option')
         self.sigma = volatility
         self.k = start_strike_price
         self.r = interest_rate
@@ -69,7 +68,12 @@ class MLCallOptions(Integrand):
         self.b = .85*self.k
         self.leveltype = 'adaptive-multi'
         self.g_submodule = getattr(self,'_g_'+self.option)
+        self.dprime = 1
+        self.level = _level
+        self.max_level = inf
         super(MLCallOptions,self).__init__()
+        #if self.discrete_distrib.low_discrepancy and self.option=='asian':
+        #    raise ParameterError('MLCallOptions does not support LD sequence for Asian Option')
 
     def get_exact_value(self):
         """ Print exact analytic value, based on s0=k. """
@@ -81,7 +85,7 @@ class MLCallOptions(Integrand):
             print('Exact value unknown for asian option')
             val = None
         elif self.option == 'lookback':
-            kk = .5*self.sigma**2/r
+            kk = .5*self.sigma**2/self.r
             val = self.k*( norm.cdf(d1) - norm.cdf(-d1)*kk -
                       exp(-self.r*self.t)*(norm.cdf(d2) - norm.cdf(d2)*kk) )
         elif self.option == 'digital':
@@ -95,13 +99,12 @@ class MLCallOptions(Integrand):
                      exp(-self.r*self.t)*norm.cdf(d4) ) )
         return val
 
-    def _g_european(self, samples, l, n, d, nf, nc, hf, hc, xf, xc):
+    def _g_european(self, t, n, d, nf, nc, hf, hc, xf, xc):
         """
         Implementation for European call option.
 
         Args:
-            samples (ndarray): nxd array of samples
-            l (int): level
+            t (ndarray): nxd array of samples
             n (int): number of samples
             d (int): number of dimensions
             nf (int): n fine samples = 2**level
@@ -116,8 +119,8 @@ class MLCallOptions(Integrand):
                 First, an ndarray of payoffs from fine paths. \
                 Second, an ndarray of payoffs from coarse paths.
         """
-        dwf = samples * sqrt(hf)
-        if l == 0:
+        dwf = t * sqrt(hf)
+        if self.level == 0:
             dwf = dwf.squeeze()
             xf = xf + self.r*xf*hf + self.sigma*xf*dwf + .5*self.sigma**2*xf*(dwf**2-hf)
         else:
@@ -131,13 +134,12 @@ class MLCallOptions(Integrand):
         pc = maximum(0,xc-self.k)
         return pf,pc
 
-    def _g_asian(self, samples, l, n, d, nf, nc, hf, hc, xf, xc):
+    def _g_asian(self, t, n, d, nf, nc, hf, hc, xf, xc):
         """
         Implementation for Asian call option.
 
         Args:
-            samples (ndarray): nxd array of samples
-            l (int): level
+            t (ndarray): nxd array of samples
             n (int): number of samples
             d (int): number of dimensions
             nf (int): n fine samples = 2**level
@@ -154,9 +156,9 @@ class MLCallOptions(Integrand):
         """
         af = .5*hf*xf
         ac = .5*hc*xc
-        dwf = sqrt(hf) * samples[:,:int(d/2)]
-        dif = sqrt(hf/12) * hf * samples[:,int(d/2):]
-        if l == 0:
+        dwf = sqrt(hf) * t[:,:int(d/2)]
+        dif = sqrt(hf/12) * hf * t[:,int(d/2):]
+        if self.level == 0:
             dwf = dwf.squeeze()
             dif = dif.squeeze()
             xf0 = xf
@@ -183,27 +185,27 @@ class MLCallOptions(Integrand):
         pc = maximum(0,ac-self.k)
         return pf,pc
 
-    def g(self, samples, l):
+    def g(self, t):
         """
         Args:
-            samples (ndarray): Gaussian(0,1^2) samples
-            l (int): level
+            t (ndarray): Gaussian(0,1^2) samples
+
         Returns:
             tuple: \
                 First, an ndarray of length 6 vector of summary statistic sums. \
                 Second, a float of cost on this level.
         """
-        n,d = samples.shape
-        nf = 2**l # n fine
+        n,d = t.shape
+        nf = 2**self.level # n fine
         nc = float(nf)/2 # n coarse
         hf = self.t/nf # timestep fine
         hc = self.t/nc # timestep coarse
         xf = tile(self.k,int(n))
         xc = xf
-        pf,pc = self.g_submodule(samples, l, n, d, nf, nc, hf, hc, xf, xc)
+        pf,pc = self.g_submodule(t, n, d, nf, nc, hf, hc, xf, xc)
         dp = exp(-self.r*self.t)*(pf-pc)
         pf = exp(-self.r*self.t)*pf
-        if l == 0:
+        if self.level == 0:
             dp = pf
         sums = zeros(6)
         sums[0] = dp.sum()
@@ -213,11 +215,23 @@ class MLCallOptions(Integrand):
         sums[4] = pf.sum()
         sums[5] = (pf**2).sum()
         cost = n*nf # cost defined as number of fine timesteps
-        return sums,cost
+        self.cost = cost
+        self.sums = sums
+        return dp
 
-    def _dim_at_level(self, l):
+    def _dimension_at_level(self, level):
         """ See abstract method. """
         if self.option == 'european':
-            return 2**l
+            return 2**level
         elif self.option == 'asian':
-            return 2**(l+1)
\ No newline at end of file
+            return 2**(level+1)
+    
+    def _spawn(self, level, sampler):
+        return MLCallOptions(
+            sampler = sampler,
+            option = self.option,
+            volatility = self.sigma,
+            start_strike_price = self.k,
+            interest_rate = self.r,
+            t_final = self.t,
+            _level = level)
