@@ -2,8 +2,11 @@ from .cub_mc_clt import CubMCCLT
 from .cub_qmc_clt import CubQMCCLT
 from .cub_mc_g import CubMCG
 from .cub_qmc_lattice_g import CubQMCLatticeG
-from .cub_qmc_sobol_g import CubQMCSobolG
+from .cub_qmc_net_g import CubQMCNetG, CubQMCSobolG
 from .cub_mc_ml import CubMCML
 from .cub_qmc_ml import CubQMCML
+from .cub_mc_ml_cont import CubMCMLCont
+from .cub_qmc_ml_cont import CubQMCMLCont
 from .cub_qmc_bayes_lattice_g import CubBayesLatticeG
+from .cub_qmc_bayes_net_g import CubBayesNetG, CubBayesSobolG
 
