@@ -1,8 +1,8 @@
 from ._stopping_criterion import StoppingCriterion
 from ..accumulate_data import MeanVarData
-from ..discrete_distribution import IIDStdGaussian
-from ..true_measure import Gaussian, BrownianMotion
-from ..integrand import Keister, AsianOption
+from ..discrete_distribution import IIDStdUniform
+from ..true_measure import Gaussian, BrownianMotion, Uniform
+from ..integrand import Keister, AsianOption, CustomFun
 from ..util import MaxSamplesWarning
 from numpy import *
 from scipy.stats import norm
@@ -14,48 +14,59 @@ class CubMCCLT(StoppingCriterion):
     """
     Stopping criterion based on the Central Limit Theorem.
     
-    >>> k = Keister(Gaussian(IIDStdGaussian(2,seed=7),covariance=1./2))
-    >>> sc = CubMCCLT(k,abs_tol=.05)
+    >>> ao = AsianOption(IIDStdUniform(seed=7))
+    >>> sc = CubMCCLT(ao,abs_tol=.05)
     >>> solution,data = sc.integrate()
-    >>> solution
-    1.834674149189029
     >>> data
-    Solution: 1.8347         
-    Keister (Integrand Object)
-    IIDStdGaussian (DiscreteDistribution Object)
-        dimension       2^(1)
-        seed            7
-        mimics          StdGaussian
-    Gaussian (TrueMeasure Object)
-        mean            0
-        covariance      2^(-1)
-        decomp_type     pca
+    MeanVarData (AccumulateData Object)
+        solution        1.519
+        error_bound     0.046
+        n_total         96028
+        n               95004
+        levels          1
+        time_integrate  ...
     CubMCCLT (StoppingCriterion Object)
-        inflate         1.200
-        alpha           0.010
         abs_tol         0.050
         rel_tol         0
         n_init          2^(10)
         n_max           10000000000
-    MeanVarData (AccumulateData Object)
-        levels          1
-        solution        1.835
-        n               5826
-        n_total         6850
-        error_bound     0.050
-        confid_int      [1.785 1.885]
-        time_integrate  ...
-    >>> ac = AsianOption(
-    ...     measure = BrownianMotion(IIDStdGaussian()),
-    ...     multi_level_dimensions = [2,4,8])
-    >>> sc = CubMCCLT(ac,abs_tol=.05)
+        inflate         1.200
+        alpha           0.010
+    AsianOption (Integrand Object)
+        volatility      2^(-1)
+        call_put        call
+        start_price     30
+        strike_price    35
+        interest_rate   0
+        mean_type       arithmetic
+        dim_frac        0
+    BrownianMotion (TrueMeasure Object)
+        time_vec        1
+        drift           0
+        mean            0
+        covariance      1
+        decomp_type     PCA
+    IIDStdUniform (DiscreteDistribution Object)
+        d               1
+        entropy         7
+        spawn_key       ()
+    >>> ao = AsianOption(IIDStdUniform(seed=7),multilevel_dims=[2,4,8])
+    >>> sc = CubMCCLT(ao,abs_tol=.05)
     >>> solution,data = sc.integrate()
+    >>> dd = IIDStdUniform(1,seed=7)
+    >>> k = Keister(dd)
+    >>> cv1 = CustomFun(Uniform(dd),lambda x: sin(pi*x).sum(1))
+    >>> cv1mean = 2/pi
+    >>> cv2 = CustomFun(Uniform(dd),lambda x: (-3*(x-.5)**2+1).sum(1))
+    >>> cv2mean = 3/4
+    >>> sc1 = CubMCCLT(k,abs_tol=.05,control_variates=[cv1,cv2],control_variate_means=[cv1mean,cv2mean])
+    >>> sol,data = sc1.integrate()
+    >>> sol
+    1.381...
     """
 
-    parameters = ['inflate','alpha','abs_tol','rel_tol','n_init','n_max']
-    
     def __init__(self, integrand, abs_tol=1e-2, rel_tol=0., n_init=1024., n_max=1e10,
-                 inflate=1.2, alpha=0.01):
+        inflate=1.2, alpha=0.01, control_variates=[], control_variate_means=[]):
         """
         Args:
             integrand (Integrand): an instance of Integrand
@@ -64,7 +75,12 @@ class CubMCCLT(StoppingCriterion):
             abs_tol (float): absolute error tolerance
             rel_tol (float): relative error tolerance
             n_max (int): maximum number of samples
+            control_variates (list): list of integrand objects to be used as control variates. 
+                Control variates are currently only compatible with single level problems. 
+                The same discrete distribution instance must be used for the integrand and each of the control variates. 
+            control_variate_means (list): list of means for each control variate
         """
+        self.parameters = ['abs_tol','rel_tol','n_init','n_max','inflate','alpha']
         # Set Attributes
         self.abs_tol = float(abs_tol)
         self.rel_tol = float(rel_tol)
@@ -72,17 +88,23 @@ class CubMCCLT(StoppingCriterion):
         self.n_max = float(n_max)
         self.alpha = float(alpha)
         self.inflate = float(inflate)
+        # QMCPy Objs
         self.integrand = integrand
+        self.true_measure = self.integrand.true_measure
+        self.discrete_distrib = self.integrand.discrete_distrib
+        self.cv = control_variates
+        self.cv_mu = control_variate_means
         # Verify Compliant Construction
-        distribution = integrand.measure.distribution
         allowed_levels = ['single','fixed-multi']
-        allowed_distribs = ["IIDStdUniform", "IIDStdGaussian", "CustomIIDDistribution"]
-        super(CubMCCLT,self).__init__(distribution, integrand, allowed_levels, allowed_distribs)
+        allowed_distribs = [IIDStdUniform]
+        allow_vectorized_integrals = False
+        super(CubMCCLT,self).__init__(allowed_levels, allowed_distribs, allow_vectorized_integrals)
 
     def integrate(self):
         """ See abstract method. """
         # Construct AccumulateData Object to House Integration data
-        self.data = MeanVarData(self, self.integrand, self.n_init)
+        self.data = MeanVarData(self, self.integrand, self.true_measure, self.discrete_distrib, 
+            self.n_init, self.cv, self.cv_mu)  # house integration data
         t_start = time()
         # Pilot Sample
         self.data.update_data()
