@@ -1,6 +1,6 @@
 from ._stopping_criterion import StoppingCriterion
 from ..accumulate_data import MLMCData
-from ..discrete_distribution import IIDStdGaussian
+from ..discrete_distribution import IIDStdUniform
 from ..true_measure import Gaussian
 from ..integrand import MLCallOptions
 from ..util import MaxSamplesWarning, ParameterError, MaxLevelsWarning, ParameterWarning
@@ -14,13 +14,28 @@ class CubMCML(StoppingCriterion):
     """
     Stopping criterion based on multi-level monte carlo.
     
-    >>> mlco = MLCallOptions(Gaussian(IIDStdGaussian(seed=7)))
+    >>> mlco = MLCallOptions(IIDStdUniform(seed=7))
     >>> sc = CubMCML(mlco,abs_tol=.05)
     >>> solution,data = sc.integrate()
-    >>> solution
-    10.445441712933263
     >>> data
-    Solution: 10.4454        
+    MLMCData (AccumulateData Object)
+        solution        10.450
+        n_total         1213658
+        levels          7
+        n_level         [1.173e+06 2.369e+04 1.174e+04 3.314e+03 1.144e+03 4.380e+02 1.690e+02]
+        mean_level      [1.006e+01 1.856e-01 1.053e-01 5.127e-02 2.699e-02 1.558e-02 7.068e-03]
+        var_level       [1.958e+02 1.596e-01 4.603e-02 1.057e-02 2.978e-03 8.701e-04 2.552e-04]
+        cost_per_sample [ 1.  2.  4.  8. 16. 32. 64.]
+        alpha           0.936
+        beta            1.870
+        gamma           1.000
+        time_integrate  ...
+    CubMCML (StoppingCriterion Object)
+        rmse_tol        0.019
+        n_init          2^(8)
+        levels_min      2^(1)
+        levels_max      10
+        theta           2^(-1)
     MLCallOptions (Integrand Object)
         option          european
         sigma           0.200
@@ -28,32 +43,15 @@ class CubMCML(StoppingCriterion):
         r               0.050
         t               1
         b               85
-    IIDStdGaussian (DiscreteDistribution Object)
-        dimension       2^(6)
-        seed            7
-        mimics          StdGaussian
+        level           0
     Gaussian (TrueMeasure Object)
         mean            0
         covariance      1
-        decomp_type     pca
-    CubMCML (StoppingCriterion Object)
-        rmse_tol        0.019
-        n_init          2^(8)
-        levels_min      2^(1)
-        levels_max      10
-        theta           2^(-2)
-    MLMCData (AccumulateData Object)
-        levels          7
-        dimensions      [ 1.  2.  4.  8. 16. 32. 64.]
-        n_level         [7.804e+05 1.533e+04 6.633e+03 2.077e+03 7.560e+02 2.730e+02 9.600e+01]
-        mean_level      [1.006e+01 1.848e-01 1.014e-01 5.138e-02 2.472e-02 1.452e-02 7.657e-03]
-        var_level       [1.963e+02 1.515e-01 4.124e-02 1.109e-02 2.901e-03 6.799e-04 1.848e-04]
-        cost_per_sample [ 1.  2.  4.  8. 16. 32. 64.]
-        n_total         805984
-        alpha           0.927
-        beta            1.946
-        gamma           1.000
-        time_integrate  ...
+        decomp_type     PCA
+    IIDStdUniform (DiscreteDistribution Object)
+        d               1
+        entropy         7
+        spawn_key       ()
 
     Original Implementation:
 
@@ -66,10 +64,8 @@ class CubMCML(StoppingCriterion):
         http://people.maths.ox.ac.uk/~gilesm/files/OPRE_2008.pdf.
     """
 
-    parameters = ['rmse_tol','n_init','levels_min','levels_max','theta']
-
     def __init__(self, integrand, abs_tol=.05, alpha=.01, rmse_tol=None, n_init=256., n_max=1e10, 
-        levels_min=2., levels_max=10., alpha0=-1., beta0=-1., gamma0=-1.):
+        levels_min=2, levels_max=10, alpha0=-1., beta0=-1., gamma0=-1.):
         """
         Args:
             integrand (Integrand): integrand with multi-level g method
@@ -77,7 +73,7 @@ class CubMCML(StoppingCriterion):
             alpha (float): uncertaintly level.
                 If rmse_tol not supplied, then rmse_tol = abs_tol/norm.ppf(1-alpha/2)
             rel_tol (float): relative tolerance. Reset if supplied, ignored if not.
-                Takes priority over aboluste tolerance and alpha if supplied. 
+                Takes priority over absolute tolerance and alpha if supplied. 
             n_init (int): initial number of samples
             n_max (int): maximum number of samples
             levels_min (int): minimum level of refinement >= 2
@@ -89,6 +85,7 @@ class CubMCML(StoppingCriterion):
         Note:
             if alpha, beta, gamma are not positive, then they will be estimated
         """
+        self.parameters = ['rmse_tol','n_init','levels_min','levels_max','theta']
         if levels_min < 2:
             raise ParameterError('needs levels_min >= 2')
         if levels_max < levels_min:
@@ -104,21 +101,25 @@ class CubMCML(StoppingCriterion):
         self.n_max = float(n_max)
         self.levels_min = float(levels_min)
         self.levels_max = float(levels_max)
-        self.theta = 0.25
-        self.integrand = integrand
+        self.theta = 0.5
         self.alpha0 = alpha0
         self.beta0 = beta0
         self.gamma0 = gamma0
+        # QMCPy Objs
+        self.integrand = integrand
+        self.true_measure = self.integrand.true_measure
+        self.discrete_distrib = self.integrand.discrete_distrib
         # Verify Compliant Construction
-        distribution = integrand.measure.distribution
         allowed_levels = ['adaptive-multi']
-        allowed_distribs = ["IIDStdUniform", "IIDStdGaussian", "CustomIIDDistribution"]
-        super(CubMCML,self).__init__(distribution, integrand, allowed_levels, allowed_distribs)
-    
+        allowed_distribs = [IIDStdUniform]
+        allow_vectorized_integrals = False
+        super(CubMCML,self).__init__(allowed_levels, allowed_distribs, allow_vectorized_integrals)
+
     def integrate(self):
         """ See abstract method. """
          # Construct AccumulateData Object to House Integration Data
-        self.data = MLMCData(self, self.integrand, self.levels_min, self.n_init, self.alpha0, self.beta0, self.gamma0)
+        self.data = MLMCData(self, self.integrand, self.true_measure, self.discrete_distrib,
+            self.levels_min, self.n_init, self.alpha0, self.beta0, self.gamma0)
         t_start = time()
         while self.data.diff_n_level.sum() > 0:
             self.data.update_data()
@@ -150,16 +151,7 @@ class CubMCML(StoppingCriterion):
                             'Failed to achieve weak convergence. levels == levels_max.',
                             MaxLevelsWarning)
                     else:
-                        self.data.levels += 1
-                        self.data.dimensions = hstack((self.data.dimensions,0))
-                        self.data.var_level = hstack((self.data.var_level,
-                            self.data.var_level[-1] / 2**self.data.beta))
-                        self.data.cost_per_sample = hstack((self.data.cost_per_sample, 
-                            self.data.cost_per_sample[-1] * 2**self.data.gamma))
-                        self.data.n_level = hstack((self.data.n_level, 0.))
-                        self.data.sum_level = hstack((self.data.sum_level,
-                            zeros((2,1))))
-                        self.data.cost_level = hstack((self.data.cost_level, 0.))
+                        self.data._add_level()
                         n_samples = self._get_next_samples()
                         self.data.diff_n_level = maximum(0., n_samples-self.data.n_level)
         # finally, evaluate multilevel estimator
