@@ -1,9 +1,9 @@
 from ._stopping_criterion import StoppingCriterion
 from ..accumulate_data import MLQMCData
-from ..discrete_distribution import Lattice
+from ..discrete_distribution import DigitalNetB2,Lattice,Halton
 from ..true_measure import Gaussian
 from ..integrand import MLCallOptions
-from ..util import MaxSamplesWarning, ParameterError
+from ..util import MaxSamplesWarning, ParameterError, MaxLevelsWarning
 from numpy import *
 from scipy.stats import norm
 from time import time
@@ -14,13 +14,24 @@ class CubQMCML(StoppingCriterion):
     """
     Stopping criterion based on multi-level quasi-Monte Carlo.
 
-    >>> mlco = MLCallOptions(Gaussian(Lattice(seed=7)))
+    >>> mlco = MLCallOptions(Lattice(seed=7))
     >>> sc = CubQMCML(mlco,abs_tol=.05)
     >>> solution,data = sc.integrate()
-    >>> solution
-    10.446333077585034
     >>> data
-    Solution: 10.4463        
+    MLQMCData (AccumulateData Object)
+        solution        10.434
+        n_total         172032
+        n_level         [4096.  256.  256.  256.  256.  256.]
+        levels          6
+        mean_level      [10.053  0.183  0.102  0.054  0.028  0.014]
+        var_level       [5.699e-05 5.129e-05 2.656e-05 1.064e-05 3.466e-06 1.113e-06]
+        bias_estimate   0.007
+        time_integrate  ...
+    CubQMCML (StoppingCriterion Object)
+        rmse_tol        0.019
+        n_init          2^(8)
+        n_max           10000000000
+        replications    2^(5)
     MLCallOptions (Integrand Object)
         option          european
         sigma           0.200
@@ -28,30 +39,18 @@ class CubQMCML(StoppingCriterion):
         r               0.050
         t               1
         b               85
-    Lattice (DiscreteDistribution Object)
-        dimension       2^(6)
-        randomize       1
-        order           natural
-        seed            854306
-        mimics          StdUniform
+        level           0
     Gaussian (TrueMeasure Object)
         mean            0
         covariance      1
-        decomp_type     pca
-    CubQMCML (StoppingCriterion Object)
-        rmse_tol        0.019
-        n_init          2^(8)
-        n_max           10000000000
-        replications    2^(5)
-    MLQMCData (AccumulateData Object)
-        levels          7
-        dimensions      [ 1.  2.  4.  8. 16. 32. 64.]
-        n_level         [4096.  512.  256.  256.  256.  256.  256.]
-        mean_level      [1.006e+01 1.852e-01 1.040e-01 5.332e-02 2.751e-02 1.399e-02 6.994e-03]
-        var_level       [6.143e-05 3.535e-05 2.896e-05 1.767e-05 2.898e-06 9.919e-07 3.260e-07]
-        bias_estimate   0.007
-        n_total         188416
-        time_integrate  ...
+        decomp_type     PCA
+    Lattice (DiscreteDistribution Object)
+        d               1
+        dvec            0
+        randomize       1
+        order           natural
+        entropy         7
+        spawn_key       ()
     
     References:
         
@@ -60,9 +59,8 @@ class CubQMCML(StoppingCriterion):
         de Gruyter, 2009. http://people.maths.ox.ac.uk/~gilesm/files/radon.pdf
     """
 
-    parameters = ['rmse_tol','n_init','n_max','replications']
-
-    def __init__(self, integrand, abs_tol=.05, alpha=.01, rmse_tol=None, n_init=256., n_max=1e10, replications=32.):
+    def __init__(self, integrand, abs_tol=.05, alpha=.01, rmse_tol=None, n_init=256., n_max=1e10, 
+        replications=32., levels_min=2, levels_max=10):
         """
         Args:
             integrand (Integrand): integrand with multi-level g method
@@ -74,7 +72,10 @@ class CubQMCML(StoppingCriterion):
                 in favor of the rmse tolerance
             n_max (int): maximum number of samples
             replications (int): number of replications on each level
+            levels_min (int): minimum level of refinement >= 2
+            levels_max (int): maximum level of refinement >= Lmin
         """
+        self.parameters = ['rmse_tol','n_init','n_max','replications']
         # initialization
         if rmse_tol:
             self.rmse_tol = float(rmse_tol)
@@ -83,26 +84,33 @@ class CubQMCML(StoppingCriterion):
         self.n_init = float(n_init)
         self.n_max = float(n_max)
         self.replications = float(replications)
+        self.levels_min = levels_min
+        self.levels_max = levels_max
+        # QMCPy Objs
         self.integrand = integrand
+        self.true_measure = self.integrand.true_measure
+        self.discrete_distrib = self.integrand.discrete_distrib
         # Verify Compliant Construction
-        distribution = integrand.measure.distribution
         allowed_levels = ['adaptive-multi']
-        allowed_distribs = ["Lattice", "Sobol","Halton"]
-        super(CubQMCML,self).__init__(distribution, integrand, allowed_levels, allowed_distribs)
+        allowed_distribs = [DigitalNetB2,Lattice,Halton]
+        allow_vectorized_integrals = False
+        super(CubQMCML,self).__init__(allowed_levels, allowed_distribs, allow_vectorized_integrals)
 
     def integrate(self):
         """ See abstract method. """
         # Construct AccumulateData Object to House Integration Data
-        self.data = MLQMCData(self, self.integrand, self.n_init, self.replications)
+        self.data = MLQMCData(self, self.integrand, self.true_measure, self.discrete_distrib,
+            self.levels_min, self.levels_max, self.n_init, self.replications)
         t_start = time()
         while True:
             self.data.update_data()
-            self.data.eval_level[:] = False
             if self.data.var_level.sum() > (self.rmse_tol**2/2.):
                 # double N_l on level with largest V_l/(2^l*N_l)
-                efficient_level = argmax(self.data.cost_level)
+                efficient_level = argmax(self.data.var_cost_ratio_level)
                 self.data.eval_level[efficient_level] = True
             elif self.data.bias_estimate > (self.rmse_tol/sqrt(2.)):
+                if self.data.levels == self.levels_max + 1:
+                        warnings.warn("Failed to achieve weak convergence. levels == levels_max.", MaxLevelsWarning)
                 # add another level
                 self.data._add_level()
             else:
